package edu.uoc.bsolee.sarau.adapters

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import edu.uoc.bsolee.sarau.databinding.ItemBookingDetailTicketBinding
import edu.uoc.bsolee.sarau.model.SelectedTicket
import java.text.NumberFormat
import java.util.*


class BookDetailTicketAdapter :
    ListAdapter<SelectedTicket, BookDetailTicketAdapter.BookTicketViewHolder>(
        SelectedTicketDiffCallBack
    ) {

    companion object SelectedTicketDiffCallBack : DiffUtil.ItemCallback<SelectedTicket>() {
        override fun areItemsTheSame(oldItem: SelectedTicket, newItem: SelectedTicket): Boolean {
            return oldItem.num == newItem.num && oldItem.ticket.id == newItem.ticket.id
        }

        override fun areContentsTheSame(oldItem: SelectedTicket, newItem: SelectedTicket): Boolean {
            return oldItem == newItem
        }

    }

    class BookTicketViewHolder(
        private val binding: ItemBookingDetailTicketBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        private lateinit var item: SelectedTicket


        fun bindTo(ticket: SelectedTicket) {
            item = ticket
            val numFormat = NumberFormat.getCurrencyInstance(Locale.FRANCE)
            binding.apply {
                tvTicketName.text = item.ticket.name
                tvTicketDescription.text = item.ticket.description
                tvPrice.text = numFormat.format(item.ticket.price)
                tvNum.text = item.num.toString()
            }
        }
    }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BookTicketViewHolder {
        return BookTicketViewHolder(
            ItemBookingDetailTicketBinding.inflate(
                LayoutInflater.from(parent.context),
                parent,
                false
            )
        )
    }

    override fun onBindViewHolder(holder: BookTicketViewHolder, position: Int) {
        holder.bindTo(getItem(position))
    }


}
