package edu.uoc.bsolee.sarau.adapters

import android.app.AlertDialog
import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.core.widget.doAfterTextChanged
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import edu.uoc.bsolee.sarau.databinding.ItemBookingTicketBinding
import edu.uoc.bsolee.sarau.model.Ticket
import java.text.NumberFormat
import java.util.*


class BookTicketAdapter(
    private val context: Context,
    private val onChangeSelectorListener: TicketSelectorListener,
) : ListAdapter<Ticket, BookTicketAdapter.BookTicketViewHolder>(TicketsDiffCallBack) {

    companion object TicketsDiffCallBack : DiffUtil.ItemCallback<Ticket>() {
        override fun areItemsTheSame(oldItem: Ticket, newItem: Ticket): Boolean {
            return oldItem.id == newItem.id
        }

        override fun areContentsTheSame(oldItem: Ticket, newItem: Ticket): Boolean {
            return oldItem == newItem
        }

    }

    class BookTicketViewHolder(
        private val context: Context,
        private val binding: ItemBookingTicketBinding,
        private val onChangeSelectorListener: TicketSelectorListener
    ) : RecyclerView.ViewHolder(binding.root) {
        private lateinit var item: Ticket

        init {
            binding.etSelector.doAfterTextChanged {
                onChangeSelectorListener.onChange(it.toString().toInt(), item)
            }
            binding.etSelector.setOnClickListener {
                item.name?.let { itName -> showSelectorDialog(itName) }
            }
        }

        private fun showSelectorDialog(name: String) {
            val builder = AlertDialog.Builder(context)
            val numTickets = arrayOf("0", "1", "2", "3", "4", "5")
            builder.setTitle("Select number of $name tickets")
                .setSingleChoiceItems(numTickets, 0) { dialog, i ->
                    binding.etSelector.setText(numTickets[i])
                    dialog.dismiss()
                }.create().show()
        }


        fun bindTo(ticket: Ticket) {
            item = ticket
            val numFormat = NumberFormat.getCurrencyInstance(Locale.FRANCE)
            binding.apply {
                tvTicketName.text = item.name
                tvTicketDescription.text = item.description
                tvPrice.text = numFormat.format(item.price)
            }
        }
    }


    class TicketSelectorListener(val onChangeSelectorListener: (numTickets: Int, ticket: Ticket) -> Unit) {
        fun onChange(numTickets: Int, ticket: Ticket) = onChangeSelectorListener(numTickets, ticket)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BookTicketViewHolder {
        return BookTicketViewHolder(
            context,
            ItemBookingTicketBinding.inflate(
                LayoutInflater.from(parent.context),
                parent,
                false
            ), onChangeSelectorListener
        )
    }

    override fun onBindViewHolder(holder: BookTicketViewHolder, position: Int) {
        holder.bindTo(getItem(position))
    }


}
