package edu.uoc.bsolee.sarau.adapters

import android.app.AlertDialog
import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.databinding.ItemBookingListBinding
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.changePaidStatus
import java.text.NumberFormat
import java.util.*


class BookingListAdapter(
    private val context: Context,
    private val onChangePaidStatusListener: BookingChangStatusListener,
) : ListAdapter<Booking, BookingListAdapter.BookingListViewHolder>(BookingsDiffCallBack) {

    companion object BookingsDiffCallBack : DiffUtil.ItemCallback<Booking>() {
        override fun areItemsTheSame(oldItem: Booking, newItem: Booking): Boolean {
            return oldItem.id == newItem.id
        }

        override fun areContentsTheSame(oldItem: Booking, newItem: Booking): Boolean {
            return oldItem == newItem
        }

    }

    class BookingListViewHolder(
        private val context: Context,
        private val binding: ItemBookingListBinding,
        private val onChangePaidStatusListener: BookingChangStatusListener
    ) : RecyclerView.ViewHolder(binding.root) {
        private lateinit var item: Booking

        init {
            itemView.setOnClickListener {
                showPaymentDialog(item.paid)
            }
        }

        private fun showPaymentDialog(paid: Boolean) {
            val builder = AlertDialog.Builder(context)
            var position = if (paid) 0 else 1
            builder.setTitle(context.getString(R.string.payment_confirmation))
                .setSingleChoiceItems(R.array.payment_status, position) { _, i ->
                    position = i
                }
                .setPositiveButton(R.string.ok) { _, _ ->
                    val newPaid = position == 0 // 0 position is paid
                    setPaymentLabel(newPaid)
                    onChangePaidStatusListener.onChange(item.changePaidStatus(newPaid))
                }
            builder.create().show()
        }

        private fun setPaymentLabel(paid: Boolean) {
            if (paid) {
                binding.tvPaymentStatus.text = context.getString(R.string.paid)
                binding.tvPaymentStatus.background =
                    ContextCompat.getDrawable(context, R.drawable.label_green)
            } else {
                binding.tvPaymentStatus.text = context.getString(R.string.pending_payment)
                binding.tvPaymentStatus.background =
                    ContextCompat.getDrawable(context, R.drawable.label_red)
            }
        }


        fun bindTo(booking: Booking) {
            item = booking
            val numFormat = NumberFormat.getCurrencyInstance(Locale.FRANCE)
            binding.apply {
                tvName.text = item.name
                tvEmail.text = item.email
                tvRefValue.text = item.id
                tvPriceAmount.text = numFormat.format(item.amount)
                setPaymentLabel(item.paid)
            }
        }
    }


    class BookingChangStatusListener(val onChangePaidStatusListener: (booking: Booking) -> Unit) {
        fun onChange(booking: Booking) = onChangePaidStatusListener(booking)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BookingListViewHolder {
        return BookingListViewHolder(
            context,
            ItemBookingListBinding.inflate(
                LayoutInflater.from(parent.context),
                parent,
                false
            ), onChangePaidStatusListener
        )
    }

    override fun onBindViewHolder(holder: BookingListViewHolder, position: Int) {
        holder.bindTo(getItem(position))
    }


}
