package edu.uoc.bsolee.sarau.adapters

import android.annotation.SuppressLint
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.ImageView
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.databinding.ItemUserEventsBinding
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.utils.Utils
import java.text.SimpleDateFormat
import java.util.*

class EventUserAdapter(
    private val fragment: Fragment,
    private val itemClickListener: EventUserClickListener
) : ListAdapter<Event, EventUserAdapter.EventUserViewHolder>(EventUsersDiffCallBack) {

    companion object EventUsersDiffCallBack : DiffUtil.ItemCallback<Event>() {
        override fun areItemsTheSame(oldItem: Event, newItem: Event): Boolean {
            return oldItem.id == newItem.id
        }

        override fun areContentsTheSame(oldItem: Event, newItem: Event): Boolean {
            return oldItem == newItem
        }

    }

    class EventUserViewHolder(
        private val binding: ItemUserEventsBinding,
        private val fragment: Fragment,
        private val itemClickListener: EventUserClickListener
    ) : RecyclerView.ViewHolder(binding.root) {
        private lateinit var item: Event

        init {
            itemView.setOnClickListener {
                itemClickListener.onClick(item)
            }
        }



        fun bindTo(event: Event) {
            item = event
            binding.apply {
                tvEventName.text = item.name
                tvCity.text = item.address.city
                tvCountry.text = item.address.country
                tvStartDate.text = buildString {
                    append(Utils.setDateTime(item.startDate).first)
                    append(" - ")
                    append(Utils.setDateTime(item.startDate).second)
                }
                tvEndDate.text =
                    buildString {
                        append(Utils.setDateTime(item.endDate).first)
                        append(" - ")
                        append(Utils.setDateTime(item.endDate).second)
                    }
                val dayFormat = SimpleDateFormat("dd", Locale.getDefault())
                val monthFormat = SimpleDateFormat("MMM", Locale.getDefault())
                tvDay.text = dayFormat.format(item.startDate)
                tvMonth.text = monthFormat.format(item.startDate)

                item.image?.let {
                    Glide.with(fragment).load(it).into(ivPoster)
                } ?: ivPoster.apply {
                    setBackgroundColor(ContextCompat.getColor(fragment.requireContext(),
                        R.color.blue_google))
                    setImageDrawable(ContextCompat.getDrawable(fragment.requireContext(), R.drawable.sarau))
                    scaleType = ImageView.ScaleType.FIT_CENTER
                }
            }
        }
    }


    class EventUserClickListener(val clickListener: (event: Event) -> Unit) {
        fun onClick(event: Event) = clickListener(event)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): EventUserViewHolder {
        return EventUserViewHolder(
            ItemUserEventsBinding.inflate(
                LayoutInflater.from(parent.context),
                parent,
                false
            ), fragment, itemClickListener
        )
    }

    override fun onBindViewHolder(holder: EventUserViewHolder, position: Int) {
        holder.bindTo(getItem(position))
    }
}
