package edu.uoc.bsolee.sarau.adapters

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import edu.uoc.bsolee.sarau.databinding.ItemLastEventCardBinding
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.utils.Utils

class LastEventHostAdapter(
    private val itemClickListener: EventHostClickListener
) : ListAdapter<Event, LastEventHostAdapter.EventHostViewHolder>(EventHostsDiffCallBack) {

    companion object EventHostsDiffCallBack : DiffUtil.ItemCallback<Event>() {
        override fun areItemsTheSame(oldItem: Event, newItem: Event): Boolean {
            return oldItem.id == newItem.id
        }

        override fun areContentsTheSame(oldItem: Event, newItem: Event): Boolean {
            return oldItem == newItem
        }

    }

    class EventHostViewHolder(
        private val binding: ItemLastEventCardBinding,
        private val itemClickListener: EventHostClickListener
    ) : RecyclerView.ViewHolder(binding.root) {
        private lateinit var item: Event

        init {
            itemView.setOnClickListener {
                itemClickListener.onClick(item)
            }
        }


        fun bindTo(event: Event) {
            item = event
            binding.apply {
                tvEventName.text = item.name
                tvEventCity.text = item.address.city
                tvEventDate.text = Utils.setDateTime(item.startDate).first
            }
        }
    }


    class EventHostClickListener(val clickListener: (event: Event) -> Unit) {
        fun onClick(event: Event) = clickListener(event)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): EventHostViewHolder {
        return EventHostViewHolder(
            ItemLastEventCardBinding.inflate(
                LayoutInflater.from(parent.context),
                parent,
                false
            ), itemClickListener
        )
    }

    override fun onBindViewHolder(holder: EventHostViewHolder, position: Int) {
        holder.bindTo(getItem(position))
    }
}
