package edu.uoc.bsolee.sarau.adapters

import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.databinding.ItemNextBookingCardBinding
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.utils.Utils
import java.text.NumberFormat
import java.util.*

class NextBookingsUserAdapter(
    private val itemClickListener: BookingUserClickListener
) : ListAdapter<Booking, NextBookingsUserAdapter.BookingUserViewHolder>(BookingUsersDiffCallBack) {

    companion object BookingUsersDiffCallBack : DiffUtil.ItemCallback<Booking>() {
        override fun areItemsTheSame(oldItem: Booking, newItem: Booking): Boolean {
            return oldItem.id == newItem.id
        }

        override fun areContentsTheSame(oldItem: Booking, newItem: Booking): Boolean {
            return oldItem == newItem
        }

    }

    class BookingUserViewHolder(
        private val context: Context,
        private val binding: ItemNextBookingCardBinding,
        private val itemClickListener: BookingUserClickListener
    ) : RecyclerView.ViewHolder(binding.root) {
        private lateinit var item: Booking

        init {
            itemView.setOnClickListener {
                itemClickListener.onClick(item)
            }
        }


        fun bindTo(booking: Booking) {
            item = booking
            val numFormat = NumberFormat.getCurrencyInstance(Locale.FRANCE)
            for (ticket in item.selectedTickets){
            binding.apply {
                item.event?.let {
                    tvEventName.text = it.name
                    tvEventCity.text = it.address.city
                    tvEventDate.text = Utils.setDateTime(it.startDate).first
                }
                tvBookingAmount.text = numFormat.format(item.amount)
                if (!item.paid) {
                    tvBookingAmount.setTextColor(ContextCompat.getColor(context, R.color.red))
                }else{
                    tvBookingAmount.setTextColor(ContextCompat.getColor(context, R.color.text))
                }
                }
            }
        }
    }


    class BookingUserClickListener(val clickListener: (bookingQR: String) -> Unit) {
        fun onClick(booking: Booking) = clickListener(booking.qr)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BookingUserViewHolder {
        return BookingUserViewHolder(parent.context,
            ItemNextBookingCardBinding.inflate(
                LayoutInflater.from(parent.context),
                parent,
                false
            ), itemClickListener
        )
    }

    override fun onBindViewHolder(holder: BookingUserViewHolder, position: Int) {
        holder.bindTo(getItem(position))
    }
}
