package edu.uoc.bsolee.sarau.adapters

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import edu.uoc.bsolee.sarau.databinding.ItemAddPromoterBinding
import edu.uoc.bsolee.sarau.model.Promoter

class PromoterAdapter(
    private val saveClickListener: PromoterClickListener,
    private val deleteClickListener: PromoterClickListener
) : ListAdapter<Promoter, PromoterAdapter.PromoterViewHolder>(PromotersDiffCallBack) {

    companion object PromotersDiffCallBack : DiffUtil.ItemCallback<Promoter>() {
        override fun areItemsTheSame(oldItem: Promoter, newItem: Promoter): Boolean {
            return oldItem.id == newItem.id
        }

        override fun areContentsTheSame(oldItem: Promoter, newItem: Promoter): Boolean {
            return oldItem == newItem
        }

    }

    class PromoterViewHolder(
        private val binding: ItemAddPromoterBinding,
        private val saveClickListener: PromoterClickListener,
        private val deleteClickListener: PromoterClickListener
    ) : RecyclerView.ViewHolder(binding.root) {
        private lateinit var item: Promoter

        init {
            binding.btnSave.setOnClickListener {
                setNotEditableUi()
                saveClickListener.onClick(getPromoter())
            }

            binding.btnDelete.setOnClickListener {
                deleteClickListener.onClick(getPromoter())
            }
        }

        private fun getPromoter(): Promoter = Promoter(
            item.id, binding.etPromoterName.text.toString(),
            binding.etPromoterEmail.text.toString(),
            binding.etPromoterPhone.text.toString(),
            binding.etPromoterCode.text.toString().uppercase(),
            true
        )


        fun bindTo(promoter: Promoter) {
            item = promoter
            binding.apply {
                item.name?.let { etPromoterName.setText(it) }
                item.email?.let { etPromoterEmail.setText(it) }
                item.telephone?.let { etPromoterPhone.setText(it) }
                item.code?.let { etPromoterCode.setText(it) }
                btnEdit.setOnClickListener {
                    setEditableUi()
                }
                if (etPromoterName.text.isNullOrEmpty() ||
                    etPromoterEmail.text.isNullOrEmpty() ||
                    etPromoterPhone.text.isNullOrEmpty() ||
                    etPromoterCode.text.isNullOrEmpty()
                ){
                    setEditableUi()
                }else{
                    setNotEditableUi()
                }
            }
        }

        private fun setEditableUi() {
            binding.apply {
                etPromoterName.isEnabled = true
                etPromoterEmail.isEnabled = true
                etPromoterPhone.isEnabled = true
                etPromoterCode.isEnabled = true
                btnSave.visibility = View.VISIBLE
                btnDelete.visibility = View.VISIBLE
                btnEdit.visibility = View.GONE
            }
        }

        private fun setNotEditableUi() {
            binding.apply {
                etPromoterName.isEnabled = false
                etPromoterEmail.isEnabled = false
                etPromoterPhone.isEnabled = false
                etPromoterCode.isEnabled = false
                btnSave.visibility = View.GONE
                btnDelete.visibility = View.GONE
                btnEdit.visibility = View.VISIBLE
            }
        }
    }


    class PromoterClickListener(val clickListener: (promoter: Promoter) -> Unit) {
        fun onClick(promoter: Promoter) = clickListener(promoter)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): PromoterViewHolder {
        return PromoterViewHolder(
            ItemAddPromoterBinding.inflate(
                LayoutInflater.from(parent.context),
                parent,
                false
            ), saveClickListener, deleteClickListener
        )
    }

    override fun onBindViewHolder(holder: PromoterViewHolder, position: Int) {
        holder.bindTo(getItem(position))
    }
}
