package edu.uoc.bsolee.sarau.repository

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.android.gms.auth.api.signin.GoogleSignInAccount
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.GoogleAuthProvider
import edu.uoc.bsolee.sarau.model.User


class AuthRepository {
    private val firebaseAuth: FirebaseAuth = FirebaseAuth.getInstance()
    private val userLiveData: MutableLiveData<User?> = MutableLiveData(null)
    private val loggedOutLiveData: MutableLiveData<Boolean> = MutableLiveData()

/*    init {
//        if (firebaseAuth.currentUser != null) {
//            val currentUser = firebaseAuth.currentUser.let {
//                User(
//                    it?.uid ?: "",
//                    it?.displayName,
//                    it?.email ?: "",
//                    it?.phoneNumber,
//                    it?.photoUrl.toString()
//                )
//            }
//            userLiveData.postValue(currentUser)
//            loggedOutLiveData.postValue(false)
//        }
    }*/

    fun login(account: GoogleSignInAccount) {
        val credential = GoogleAuthProvider.getCredential(account.idToken, null)

        FirebaseAuth.getInstance().signInWithCredential(credential).addOnCompleteListener {
            if (it.isSuccessful) {
                val user = User(
                    account.id!!,
                    account.displayName,
                    account.email!!,
                    "",
                    account.photoUrl.toString()
                )
                userLiveData.postValue(user)
                loggedOutLiveData.postValue(false)
            } else {
                userLiveData.postValue(null)
            }
        }
    }


    fun logOut() {
        // Firebase Sign Out
        firebaseAuth.signOut()
        loggedOutLiveData.postValue(true)
    }

    fun getUserLiveData(): LiveData<User?> {
        return userLiveData
    }

    fun getLoggedOutLiveData(): LiveData<Boolean> {
        return loggedOutLiveData
    }
}