package edu.uoc.bsolee.sarau.repository

import androidx.lifecycle.MutableLiveData
import com.google.firebase.firestore.FieldValue
import com.google.firebase.firestore.FirebaseFirestore
import com.google.firebase.firestore.SetOptions
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.model.Promoter

class BookingRepository {
    companion object {
        const val COLLECTION_EVENTS = "Events"
        const val SUBCOLLECTION_TICKETS = "Tickets"
        const val SUBCOLLECTION_PROMOTERS = "Promoters"
        const val SUBCOLLECTION_BOOKINGS = "Bookings"
        const val FIELD_CAPACITY = "capacity"
    }

    private val db = FirebaseFirestore.getInstance()
    private val bookingSaved = MutableLiveData<Booking>()
    private var bookingsFromPromoterLiveData = MutableLiveData<List<Booking>>()
    private var eventBookingsLiveData = MutableLiveData<List<Booking>>()


    fun getBookingsFromPromoter(event: Event, promoter: Promoter) {
        db.collection(COLLECTION_EVENTS).document(event.id)
            .collection(SUBCOLLECTION_PROMOTERS).document(promoter.id)
            .collection(SUBCOLLECTION_BOOKINGS).get().addOnSuccessListener {
                val documents = it.documents
                val bookings = mutableListOf<Booking>()
                for (doc in documents) {
                    val booking = doc.toObject(Booking::class.java)
                    booking?.let { itBooking -> bookings.add(itBooking) }
                }
                bookingsFromPromoterLiveData.value = bookings
            }
    }

    fun getBookingsFromPromoterLiveData() = bookingsFromPromoterLiveData

    fun loadEventBookings(event: Event, promoters: List<Promoter>) {
        val bookings = mutableListOf<Booking>()
        for (promoter in promoters) {
            db.collection(EventRepository.COLLECTION_EVENTS).document(event.id)
                .collection(SUBCOLLECTION_PROMOTERS).document(promoter.id)
                .collection(SUBCOLLECTION_BOOKINGS).get().addOnSuccessListener {
                    val documents = it.documents
                    for (doc in documents) {
                        val booking = doc.toObject(Booking::class.java)
                        booking?.let { itBooking -> bookings.add(itBooking) }
                    }
                    eventBookingsLiveData.value = bookings
                }
        }
    }

    fun getEventBookingsLiveData() = eventBookingsLiveData

    fun saveBookingToDB(booking: Booking) {
        db.collection(COLLECTION_EVENTS).document(booking.eventId)
            .collection(SUBCOLLECTION_PROMOTERS).document(booking.promoterId).collection(
                SUBCOLLECTION_BOOKINGS
            ).document(booking.qr).set(booking).addOnSuccessListener {
                decreaseTickets(booking)
                bookingSaved.value = booking
            }
    }

    private fun decreaseTickets(booking: Booking) {
        val ticketsRef = db.collection(COLLECTION_EVENTS).document(booking.eventId).collection(
            SUBCOLLECTION_TICKETS
        )
        for (ticket in booking.selectedTickets) {
            val num = ticket.num * -1L
            ticketsRef.document(ticket.ticket.id).update(FIELD_CAPACITY, FieldValue.increment(num))
        }
    }

    fun updateBooking(booking: Booking) {
        db.collection(COLLECTION_EVENTS).document(booking.eventId)
            .collection(SUBCOLLECTION_PROMOTERS).document(booking.promoterId)
            .collection(SUBCOLLECTION_BOOKINGS).document(booking.qr)
            .set(booking, SetOptions.merge()).addOnSuccessListener {
                updateEventBookings(booking)
            }
    }

    private fun updateEventBookings(booking: Booking) {
        val oldBookings = eventBookingsLiveData.value
        val newBookings = mutableListOf<Booking>()
        oldBookings?.let { newBookings.addAll(it.filter { itBooking -> itBooking.promoterId != booking.promoterId }) } // take only bookings from promoter not updated
        db.collection(COLLECTION_EVENTS).document(booking.eventId)
            .collection(SUBCOLLECTION_PROMOTERS).document(booking.promoterId)
            .collection(SUBCOLLECTION_BOOKINGS).get().addOnSuccessListener {
                val documents = it.documents
                for (doc in documents) {
                    val newBooking = doc.toObject(Booking::class.java)
                    newBooking?.let { itNewBooking -> newBookings.add(itNewBooking) }
                }
                eventBookingsLiveData.value = newBookings
            }
    }

    // endregion
}