package edu.uoc.bsolee.sarau.repository

import android.graphics.Bitmap
import android.net.Uri
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.firebase.firestore.FirebaseFirestore
import com.google.firebase.firestore.SetOptions
import com.google.firebase.ktx.Firebase
import com.google.firebase.storage.ktx.storage
import edu.uoc.bsolee.sarau.model.Address
import edu.uoc.bsolee.sarau.model.Host
import java.io.ByteArrayOutputStream

class HostRepository {
    companion object {
        const val COLLECTION_HOSTS = "Hosts"
    }

    private val db = FirebaseFirestore.getInstance()
    private val storage = Firebase.storage
    private val hostLiveData = MutableLiveData<Host>()
    private val saveHostIsSuccessful = MutableLiveData<Boolean>()
    private val imageUriLiveData = MutableLiveData<Uri>()

    fun saveHost(host: Host) {
        // Check if Host Exists
        db.collection(COLLECTION_HOSTS).document(host.email).get().addOnSuccessListener {
            if (it.data != null) {
//            Update only non existing fields
                val updateHost = mutableMapOf<String, String>()
                if ((it.get("id") as String?).isNullOrEmpty()) {
                    updateHost["id"] = host.id
                }
                if ((it.get("contactName") as String?).isNullOrEmpty()) {
                    host.contactName?.let { itContactName ->
                        updateHost.put(
                            "contactName",
                            itContactName
                        )
                    }
                }
                if ((it.get("email") as String?).isNullOrEmpty()) {
                    updateHost["email"] = host.id
                }
                if ((it.get("telephone") as String?).isNullOrEmpty()) {
                    host.telephone?.let { itTelephone -> updateHost.put("telephone", itTelephone) }
                }
                if ((it.get("image") as String?).isNullOrEmpty()) {
                    host.image?.let { itImage -> updateHost.put("image", itImage) }
                }
                db.collection(COLLECTION_HOSTS).document(host.email)
                    .set(updateHost, SetOptions.merge()).addOnSuccessListener {
                        saveHostIsSuccessful.postValue(true)
                    }
            } else {
                db.collection(COLLECTION_HOSTS).document(host.email).set(host)
                    .addOnSuccessListener {
                        saveHostIsSuccessful.postValue(true)
                    }
            }
        }

    }

    fun updateOrSetHost(host: Host) {
        db.collection(COLLECTION_HOSTS).document(host.email)
            .set(host, SetOptions.merge())
    }


    fun getHost(email: String) {
        db.collection(COLLECTION_HOSTS).document(email).get().addOnSuccessListener {
            val host = Host(
                it.get("id") as String,
                it.get("businessName") as String?,
                it.get("contactName") as String?,
                it.get("email") as String,
                it.get("telephone") as String?,
                Address(
                    it.get("address.street") as String?, it.get("address.postalCode") as String?,
                    it.get("address.city") as String?, it.get("address.country") as String?
                ),
                it.get("vatNumber") as String?,
                it.get("image") as String?,
            )
            hostLiveData.value = host
        }
    }

    fun getHostLiveData(): LiveData<Host> = hostLiveData

    fun getSaveHostIsSuccessful(): LiveData<Boolean> = saveHostIsSuccessful

    fun saveImageToStorage(email: String, image: Bitmap){
        val imageRef = storage.reference.child("hosts/$email/profile.jpg")
        val baos = ByteArrayOutputStream()
        image.compress(Bitmap.CompressFormat.JPEG, 100, baos)
        val data = baos.toByteArray()

        val uploadTask = imageRef.putBytes(data)

        uploadTask.continueWithTask { task ->
            if (!task.isSuccessful) {
                task.exception?.let {
                    throw it
                }
            }
            imageRef.downloadUrl
        }.addOnCompleteListener { task ->
            if (task.isSuccessful) {
                val downloadUri = task.result
                imageUriLiveData.postValue(downloadUri)
            }
        }

    }

    fun getImageUriLiveData(): LiveData<Uri> = imageUriLiveData

    fun updateImageToDB(email: String, uri: Uri) {
        db.collection(COLLECTION_HOSTS).document(email).update("image", uri.toString()).addOnSuccessListener {
            getHost(email)
        }
    }


}
