package edu.uoc.bsolee.sarau.repository

import android.graphics.Bitmap
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.firebase.firestore.FirebaseFirestore
import com.google.firebase.firestore.SetOptions
import com.google.firebase.ktx.Firebase
import com.google.firebase.storage.ktx.storage
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.User
import edu.uoc.bsolee.sarau.model.changeImageUri
import java.io.ByteArrayOutputStream

class UserRepository {
    companion object {
        const val COLLECTION_USERS = "Users"
        const val SUBCOLLECTION_BOOKINGS = "Bookings"
    }

    private val db = FirebaseFirestore.getInstance()
    private val storage = Firebase.storage
    private val usersLiveData = MutableLiveData<User>()
    private val saveUserIsSuccessful = MutableLiveData<Boolean>()
    private val bookingSaved = MutableLiveData<Booking?>(null)
    private val bookingFromQRLiveData = MutableLiveData<Booking>()
    private val userBookingsLiveData = MutableLiveData<List<Booking>>()

    fun saveUser(user: User) {
        // Check if User Exists
        db.collection(COLLECTION_USERS).document(user.email).get().addOnSuccessListener {
            if (it.data != null) {
//            Update only non existing fields
                val updateHost = mutableMapOf<String, String>()
                if ((it.get("id") as String?).isNullOrEmpty()) {
                    updateHost["id"] = user.id
                }
                if ((it.get("name") as String?).isNullOrEmpty()) {
                    user.name?.let { itContactName ->
                        updateHost.put(
                            "name",
                            itContactName
                        )
                    }
                }
                if ((it.get("email") as String?).isNullOrEmpty()) {
                    updateHost["email"] = user.id
                }
                if ((it.get("telephone") as String?).isNullOrEmpty()) {
                    user.telephone?.let { itTelephone -> updateHost.put("telephone", itTelephone) }
                }
                if ((it.get("image") as String?).isNullOrEmpty()) {
                    user.image?.let { itImage -> updateHost.put("image", itImage) }
                }
                db.collection(COLLECTION_USERS).document(user.email)
                    .set(updateHost, SetOptions.merge()).addOnSuccessListener {
                        saveUserIsSuccessful.postValue(true)
                    }
            } else {
                db.collection(COLLECTION_USERS).document(user.email).set(user)
                    .addOnSuccessListener {
                        saveUserIsSuccessful.postValue(true)
                    }
            }
        }
    }

    fun saveOrUpdateUser(userEdited: User) {
        db.collection(COLLECTION_USERS).document(userEdited.email)
            .set(userEdited, SetOptions.merge()).addOnSuccessListener {
                saveUserIsSuccessful.postValue(true)
                getUser(userEdited.email)
            }
    }

    fun updateImageAndSave(user: User, image: Bitmap) {
        // Save image
        val imageRef = storage.reference.child("${COLLECTION_USERS}/${user.email}/image.jpg")
        val baos = ByteArrayOutputStream()
        image.compress(Bitmap.CompressFormat.JPEG, 100, baos)
        val data = baos.toByteArray()

        val uploadTask = imageRef.putBytes(data)

        uploadTask.continueWithTask { task ->
            if (!task.isSuccessful) {
                task.exception?.let {
                    throw it
                }
            }
            imageRef.downloadUrl
        }.addOnCompleteListener { task ->
            if (task.isSuccessful) {
                val downloadUri = task.result
                saveOrUpdateUser(user.changeImageUri(downloadUri.toString()))
            }
        }
    }


    fun getUser(email: String) {
        db.collection(COLLECTION_USERS).document(email).get().addOnSuccessListener {
            val user = User(
                it.get("id") as String,
                it.get("name") as String?,
                it.get("email") as String,
                it.get("telephone") as String?,
                it.get("image") as String?
            )

            usersLiveData.value = user
        }
    }

    fun getUserLiveData(): LiveData<User> = usersLiveData

    fun getSaveUserIsSuccessful(): LiveData<Boolean> = saveUserIsSuccessful

    fun saveBookingToUser(booking: Booking) {
        db.collection(COLLECTION_USERS).document(booking.email)
            .collection(SUBCOLLECTION_BOOKINGS)
            .document(booking.qr).set(booking).addOnSuccessListener {
                bookingSaved.value = booking
            }
    }

    fun getBookingSaved() = bookingSaved
    fun resetBookingSaved(){
        bookingSaved.value = null
    }

    fun getBookingFromId(qr: String) {
        db.collection(COLLECTION_USERS).document(AppSarau.prefs.getUser())
            .collection(SUBCOLLECTION_BOOKINGS).document(qr).get()
            .addOnSuccessListener { document ->
                val booking = document.toObject(Booking::class.java)
                booking?.let { bookingFromQRLiveData.value = it }
            }
    }

    fun getBookingFromIdLiveData() = bookingFromQRLiveData

    fun getBookingsFromUser() {
        db.collection(COLLECTION_USERS).document(AppSarau.prefs.getUser()).collection(
            SUBCOLLECTION_BOOKINGS
        ).get().addOnSuccessListener {
            val documents = it.documents
            val bookings = mutableListOf<Booking>()
            for (doc in documents) {
                val booking = doc.toObject(Booking::class.java)
                booking?.let { itBooking -> bookings.add(itBooking) }
            }
            userBookingsLiveData.value = bookings
        }
    }

    fun getBookingsFromUserLiveData() = userBookingsLiveData

    fun updateBooking(booking: Booking) {
        db.collection(COLLECTION_USERS).document(booking.email)
            .collection(SUBCOLLECTION_BOOKINGS)
            .document(booking.qr).set(booking, SetOptions.merge()).addOnSuccessListener {
                getBookingsFromUser() // refresh bookings
            }
    }


}