package edu.uoc.bsolee.sarau.ui.host.booking_list

import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.AdapterView
import android.widget.ArrayAdapter
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.adapters.BookingListAdapter
import edu.uoc.bsolee.sarau.databinding.ActivityBookingListBinding
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.Promoter
import java.text.NumberFormat
import java.util.*

class BookingListActivity : AppCompatActivity() {

    companion object {
        const val EVENT_ID = "eventId"
    }

    // Argument
    private lateinit var eventId: String

    // View Binding
    private var _binding: ActivityBookingListBinding? = null
    val binding: ActivityBookingListBinding get() = _binding!!

    // ViewModel
    private val viewModel: BookingListViewModel by viewModels { BookingListViewModelFactory() }

    // Adapter
    private lateinit var bookingListAdapter: BookingListAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        eventId = intent.getStringExtra(EVENT_ID).toString()
        actionBar?.setDisplayHomeAsUpEnabled(true)
        _binding = ActivityBookingListBinding.inflate(layoutInflater)
        initUI()
        initObservers()
        setContentView(binding.root)
    }

    override fun onResume() {
        super.onResume()
        viewModel.loadEvent(eventId)
    }

    private fun initUI() {
        // Recycler View
        bookingListAdapter =
            BookingListAdapter(this, BookingListAdapter.BookingChangStatusListener {
                viewModel.updateBookingToDB(it)
                Log.d("BookingListActivity.Paid", "${it.name}-Paid:${it.paid}")
            })

        binding.rvBookings.layoutManager = LinearLayoutManager(this)
        binding.rvBookings.adapter = bookingListAdapter


    }


    private fun initObservers() {
        viewModel.promoters.observe(this) {
            viewModel.loadBookings(it)
            setupSpinner(it)
        }

        viewModel.bookings.observe(this) {
            populateUI(it)
            bookingListAdapter.submitList(it.sortedBy { booking -> booking.dateCreated })
        }

    }

    private fun populateUI(list: List<Booking>) {
        val numFormat = NumberFormat.getCurrencyInstance(Locale.FRANCE)
        var totalTickets = 0
        var totalPaid = 0.0
        var pendingPayment = 0.0
        for (booking in list) {
            if (booking.paid) {
                totalPaid += booking.amount
            } else {
                pendingPayment += booking.amount
            }
            for (ticket in booking.selectedTickets) {
                totalTickets += ticket.num
            }
        }
        title = viewModel.event.value?.name ?: getString(R.string.booking_list)
        binding.apply {
            tvTotalTicketsValue.text = totalTickets.toString()
            tvPaidValue.text = numFormat.format(totalPaid)
            tvPendingValue.text = numFormat.format(pendingPayment)
        }
    }

    private fun setupSpinner(promoters: List<Promoter>) {
        val list = mutableListOf<String>()
        list.add(getString(R.string.all_promoters))
        for (promoter in promoters) {
            list.add(promoter.code.toString())
        }
        val spinnerAdapter = ArrayAdapter(baseContext, android.R.layout.simple_spinner_item, list)
        binding.spnPromoterSelector.adapter = spinnerAdapter

        binding.spnPromoterSelector.onItemSelectedListener =
            object : AdapterView.OnItemSelectedListener {
                override fun onItemSelected(
                    p0: AdapterView<*>?,
                    p1: View?,
                    position: Int,
                    p3: Long
                ) {
                    viewModel.updatePromotersList(position, list[position])
                }

                override fun onNothingSelected(p0: AdapterView<*>?) {

                }
            }
    }
}
