package edu.uoc.bsolee.sarau.ui.host.booking_list

import androidx.lifecycle.LiveData
import androidx.lifecycle.ViewModel
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.model.Promoter
import edu.uoc.bsolee.sarau.repository.BookingRepository
import edu.uoc.bsolee.sarau.repository.EventRepository
import edu.uoc.bsolee.sarau.repository.UserRepository

class BookingListViewModel(
    private val bookingRepository: BookingRepository,
    private val eventRepository: EventRepository,
    private val userRepository: UserRepository
) : ViewModel() {

    private val _event = eventRepository.getEventLiveData()
    val event : LiveData<Event> get() = _event

    private val _promoters = eventRepository.getPromotersLiveData()
    val promoters : LiveData<List<Promoter>> get() = _promoters

    private val _bookings = bookingRepository.getEventBookingsLiveData()
    val bookings : LiveData<List<Booking>> get() = _bookings

    fun loadEvent(eventId: String) {
        eventRepository.getEventFromId(eventId)
    }

    fun loadBookings(promoters: List<Promoter>) {
        event.value?.let { bookingRepository.loadEventBookings(it,promoters) }
    }

    fun updateBookingToDB(booking: Booking) {
        bookingRepository.updateBooking(booking)
        userRepository.updateBooking(booking)
    }

    fun updatePromotersList(position: Int, code: String) {
        val promotersList = promoters.value
        promotersList?.let { list ->
            if (position == 0){ // All promoters
                loadBookings(list)
            }else{
                loadBookings(list.filter { promoter ->
                    promoter.code == code
                })
            }
        }

    }

}