package edu.uoc.bsolee.sarau.ui.host.check_in

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.model.Promoter
import edu.uoc.bsolee.sarau.repository.BookingRepository
import edu.uoc.bsolee.sarau.repository.EventRepository
import edu.uoc.bsolee.sarau.repository.UserRepository

class EventCheckInViewModel(
    private val bookingRepository: BookingRepository,
    private val eventRepository: EventRepository,
    private val userRepository: UserRepository
) : ViewModel() {

    private var eventId: String = ""

    private val _correctCheckIn = MutableLiveData<Booking?>(null)
    val correctCheckIn : LiveData<Booking?> get() = _correctCheckIn

    private val _checkedIn = MutableLiveData<Booking?>(null)
    val checkedIn : LiveData<Booking?> get() = _checkedIn

    private val _bookingNotExist = MutableLiveData(false)
    val bookingNotExist : LiveData<Boolean> get() = _bookingNotExist

    private val _qrCode = MutableLiveData<String?>(null)
    val qrCode : LiveData<String?> get() = _qrCode

    private val _event = eventRepository.getEventLiveData()
    val event : LiveData<Event> get() = _event

    private val _promoters = eventRepository.getPromotersLiveData()
    val promoters : LiveData<List<Promoter>> get() = _promoters

    private val _bookings = bookingRepository.getEventBookingsLiveData()
    val bookings : LiveData<List<Booking>> get() = _bookings

    fun loadEvent(eventId: String) {
        this.eventId = eventId
        eventRepository.getEventFromId(this.eventId)
    }

    fun loadBookings(promoters: List<Promoter>) {
        event.value?.let { bookingRepository.loadEventBookings(it,promoters) }
    }

    fun updateBookingToDB(booking: Booking) {
        bookingRepository.updateBooking(booking)
        userRepository.updateBooking(booking)
    }



    fun checkScanResult(result: String) {
    val eventBookings = bookings.value
        if (eventBookings != null) {
            var bookingNotExist = true
            for (booking in eventBookings){
                if(booking.qr == result && !booking.checkedIn){
                    _correctCheckIn.value = booking
                }
                if(booking.qr == result && booking.checkedIn){
                    _checkedIn.value = booking
                }

                if (booking.qr == result){
                    bookingNotExist = false
                }
            }
            _bookingNotExist.value = bookingNotExist
        }
    }

    fun resetCheckIn(){
        _correctCheckIn.value = null
        _checkedIn.value = null
        _bookingNotExist.value = false
    }

    fun setQR(contents: String) {
        _qrCode.value = contents
    }
    fun resetQr(){
        _qrCode.value = null
    }

}