package edu.uoc.bsolee.sarau.ui.host.detail_event

import android.graphics.Bitmap
import android.net.Uri
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.model.Promoter
import edu.uoc.bsolee.sarau.model.Ticket
import edu.uoc.bsolee.sarau.model.inactivate
import edu.uoc.bsolee.sarau.repository.EventRepository
import java.util.*

class HostDetailEventViewModel(private val eventRepository: EventRepository) : ViewModel() {

    private val _event = eventRepository.getEventLiveData()
    val event get() = _event

    private var ticketsList = mutableListOf<Ticket>()
    private var savedTickets = mutableListOf<Ticket>()
    private val _tickets = MutableLiveData<List<Ticket>>()
    val tickets get() = _tickets
    private val _initialTickets = eventRepository.getTicketsLiveData()
    val initialTickets get() = _initialTickets

    private var promotersList = mutableListOf<Promoter>()
    private var savedPromoters = mutableListOf<Promoter>()
    private val _promoters = MutableLiveData<List<Promoter>>()
    val promoters get() = _promoters
    private val _initialPromoters = eventRepository.getPromotersLiveData()
    val initialPromoters get() = _initialPromoters


    private val _imageTempUri = MutableLiveData<Uri>()
    val imageTempUri: LiveData<Uri>
        get() = _imageTempUri

    private var _imageBitmap: Bitmap? = null
    private val imageBitmap: Bitmap?
        get() = _imageBitmap

    fun selectImage(result: Uri?) {
        result?.let { _imageTempUri.value = it }
    }

    fun setPosterBitmap(bitmap: Bitmap) {
        _imageBitmap = bitmap
    }

    fun setLocalUri(event: Event?) {
        event?.image?.let { _imageTempUri.value = Uri.parse(it) }
    }


    // region Tickets
    fun setTicketsList(tickets: List<Ticket>?) {
        tickets?.let {
            ticketsList.addAll(it)
            savedTickets.addAll(it)
            _tickets.value = ticketsList
        }
    }

    fun ticketsListSize() = ticketsList.size

    fun addTicket(ticket: Ticket) {
        ticketsList.add(ticket)
        _tickets.value = ticketsList
    }

    fun saveTicket(ticket: Ticket) {
        var isNew = true
        val newSavedTicketsList = savedTickets.map {
            if (it.id == ticket.id) {
                isNew = false
                return@map ticket
            } else {
                return@map it
            }
        } as MutableList<Ticket>

        if (isNew) newSavedTicketsList.add(ticket)

        savedTickets = newSavedTicketsList
    }

    fun deleteTicket(idTicket: String) {
        val newTicketsList = ticketsList.map {
            if (it.id == idTicket) {
                return@map it.inactivate()
            }
            return@map it
        } as MutableList<Ticket>
        ticketsList = newTicketsList
        _tickets.value = ticketsList
    }

    // endregion

    // region Promoter
    fun setPromotersList(promoters: List<Promoter>?) {
        promoters?.let {
            promotersList.addAll(it)
            savedPromoters.addAll(it)
            _promoters.value = promotersList
        }
    }

    fun promotersListSize() = promotersList.size

    fun addPromoter(promoter: Promoter) {
        promotersList.add(promoter)
        _promoters.value = promotersList
    }

    fun savePromoter(promoter: Promoter) {
        var isNew = true
        val newSavedPromotersList = savedPromoters.map {
            if (it.id == promoter.id) {
                isNew = false
                return@map promoter
            } else {
                return@map it
            }
        } as MutableList<Promoter>

        if (isNew) {
            newSavedPromotersList.add(promoter)
        }
        savedPromoters = newSavedPromotersList
    }

    fun deletePromoter(idPromoter: String) {
        val newPromotersList = promotersList.map {
            if (it.id == idPromoter) {
                return@map it.inactivate()
            }
            return@map it
        } as MutableList<Promoter>
        promotersList = newPromotersList
        _promoters.value = promotersList
    }

    fun initPopulateUI(eventId: String) {
        eventRepository.getEventFromId(eventId)
    }

    // region Dates
    private val _startDate = MutableLiveData<Date?>(null)
    val startDate: LiveData<Date?>
        get() = _startDate


    fun setInitialDates(event: Event) {
        _startDate.value = event.startDate
        _endDate.value = event.endDate
    }

    fun setStartDate(timestamp: Long?) {
        _startDate.value = timestamp?.let { Date(it) }
    }

    fun setStartTime(hour: Int, minute: Int) {
        val dateInMillis = startDate.value?.time
        val dateTimeInMillis = dateInMillis?.let { it + 3600000 * hour + 60000 * minute }
        _startDate.value = dateTimeInMillis?.let { Date(it) }
    }

    private val _endDate = MutableLiveData<Date?>(null)
    val endDate: LiveData<Date?>
        get() = _endDate

    fun setEndDate(timestamp: Long?) {
        _endDate.value = timestamp?.let { Date(it) }
    }

    fun setEndTime(hour: Int, minute: Int) {
        val dateInMillis = endDate.value?.time
        val dateTimeInMillis = dateInMillis?.let { it + 3600000 * hour + 60000 * minute }
        _endDate.value = dateTimeInMillis?.let { Date(it) }
    }


    // endregion

    private val _errorMessage = MutableLiveData<Int?>(null)
    val errorMessage: LiveData<Int?>
        get() = _errorMessage

    private val _validatedDraft = MutableLiveData<Event?>(null)
    val validatedDraft: LiveData<Event?>
        get() = _validatedDraft

    fun validateDraft(event: Event) {
        // Name and statDate have to be not empty
        if (event.name.isEmpty() || event.startDate == Date(0)) {
            _errorMessage.value = R.string.draft_error_message
        } else {
            _validatedDraft.value = event
        }
    }

    fun resetErrorMessage() {
        _errorMessage.value = null
    }

    fun resetValidatedDraft() {
        _validatedDraft.value = null
    }

    private val _validatedPublish = MutableLiveData<Event?>(null)
    val validatedPublish: LiveData<Event?>
        get() = _validatedPublish

    fun validatePublish(event: Event) {
        // No fields could be empty and endDate must be later than startDate
        if (!isEventAllFillIn(event)) {
            _errorMessage.value = R.string.error_message_validation_publish
        } else if (event.startDate >= event.endDate) {
            _errorMessage.value = R.string.error_message_validate_dates
        } else {
            _validatedPublish.value = event
        }
    }

    fun resetValidatePublish() {
        _validatedPublish.value = null
    }

    private fun isEventAllFillIn(event: Event): Boolean {
        return (event.name.isNotEmpty() ||
                event.description.isNotEmpty() ||
                event.name.isNotEmpty() ||
                !event.address.street.isNullOrEmpty() ||
                !event.address.postalCode.isNullOrEmpty() ||
                !event.address.city.isNullOrEmpty() ||
                !event.address.country.isNullOrEmpty() ||
                event.startDate != Date(0) ||
                event.endDate != Date(0) ||
                event.paymentMethod.name.isNotEmpty() ||
                event.paymentMethod.description.isNotEmpty() ||
                event.paymentMethod.account.isNotEmpty())
    }

    private val _isEventUpdated = eventRepository.getIsEventUpdated()
    val isEventUpdated: LiveData<Boolean>
        get() = _isEventUpdated

    fun updateEvent(event: Event) {
        eventRepository.updateEvent(
            event,
            imageBitmap,
            savedTickets as List<Ticket>,
            savedPromoters as List<Promoter>
        )
    }

    fun refreshUI(eventId: String) {
        ticketsList = mutableListOf()
        promotersList = mutableListOf()
        initPopulateUI(eventId)
    }


}