package edu.uoc.bsolee.sarau.ui.host.main_host

import android.graphics.Bitmap
import android.net.Uri
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.model.Host
import edu.uoc.bsolee.sarau.repository.AuthRepository
import edu.uoc.bsolee.sarau.repository.EventRepository
import edu.uoc.bsolee.sarau.repository.HostRepository

class MainHostViewModel(
    private val hostRepository: HostRepository,
    private val authRepository: AuthRepository,
    private val eventRepository: EventRepository
) : ViewModel() {

    init {
        getCurrentHost()
        getEvents()
    }

    private val _host = hostRepository.getHostLiveData()
    val host: LiveData<Host>
        get() = _host

    private val _nextEvents = eventRepository.getNextEventsLiveData()
    val nextEvents: LiveData<List<Event>>
        get() = _nextEvents

    private val _lastEvents = eventRepository.getLastEventsLiveData()
    val lastEvents: LiveData<List<Event>>
        get() = _lastEvents

    // URL image in Firebase Storage
    private val _imageUrl = hostRepository.getImageUriLiveData()
    val imageUrl: LiveData<Uri>
        get() = _imageUrl

    // Local Uri Image
    private val _uriImage = MutableLiveData<Uri>()
    val uriImage: LiveData<Uri>
        get() = _uriImage


    private val _isLoggedOutDone = authRepository.getLoggedOutLiveData()
    val isLoggedOutDone: LiveData<Boolean>
        get() = _isLoggedOutDone

    fun getCurrentHost() {
        hostRepository.getHost(AppSarau.prefs.getUser())
    }

    fun getEvents() {
        eventRepository.getHostNextEvents()
        eventRepository.getHostLastEvents()
    }


    fun logout() {
        authRepository.logOut()
    }

    fun clearData() {
        AppSarau.prefs.clear()
    }

    fun saveHostToDB(host: Host) {
        hostRepository.updateOrSetHost(host)
    }

    fun selectImage(uri: Uri?) {
        uri?.let { _uriImage.value = it }
    }

    fun saveImageToDB(bitmap: Bitmap?) {
        if (host.value?.email != null) {
            bitmap?.let {
                hostRepository.saveImageToStorage(host.value?.email!!, it)
            }
        }
    }

    fun updateImageOnDB(uri: Uri?) {
        if (host.value?.email != null) {
            uri?.let {
                hostRepository.updateImageToDB(host.value?.email!!, it)
                _uriImage.value = it
            }
        }
    }
}