package edu.uoc.bsolee.sarau.ui.login

import android.content.Intent
import android.os.Bundle
import android.view.View
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.databinding.ActivityLoginBinding
import edu.uoc.bsolee.sarau.model.asHost
import edu.uoc.bsolee.sarau.ui.host.main_host.MainHostActivity
import edu.uoc.bsolee.sarau.ui.user.main_user.MainUserActivity

class LoginActivity : AppCompatActivity() {

    companion object {
        const val USER_LOGIN_FRAGMENT = 100
        const val HOST_LOGIN_FRAGMENT = 101

    }

    // ViewModel
    private val viewModel: LoginViewModel by viewModels { LoginViewModelFactory() }

    private val launcher =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            viewModel.onSignInResult(result)
        }

    // View Binding
    private var _binding: ActivityLoginBinding? = null
    private val binding: ActivityLoginBinding
        get() = _binding!!


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        _binding = ActivityLoginBinding.inflate(layoutInflater)
        setContentView(binding.root)
        initUi()
        initObservers()
    }

    private fun initUi() {
        // Hide the action bar and status bar.
        window.decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_FULLSCREEN
        supportActionBar?.hide()
        binding.inclProgressBar.cvProgressBar.visibility = View.GONE

    }

    private fun initObservers() {
        viewModel.fragment.observe(this) {
            if (it == USER_LOGIN_FRAGMENT) {
                val fragment = LoginUserFragment()
                val transaction = supportFragmentManager.beginTransaction()
                transaction.replace(R.id.fragment_container, fragment)
                transaction.commit()
            } else if (it == HOST_LOGIN_FRAGMENT) {
                val fragment = LoginHostFragment()
                val transaction = supportFragmentManager.beginTransaction()
                transaction.replace(R.id.fragment_container, fragment)
                transaction.commit()
            }
        }

        viewModel.signIn.observe(this) {
            if (it) {
                binding.inclProgressBar.cvProgressBar.visibility = View.VISIBLE
                viewModel.onClickSignInDone()
                startSignIn()
            }
        }

        viewModel.userLiveData.observe(this) {
            it?.let { itUser ->
                viewModel.isHostUser.value?.let { itIsHost -> AppSarau.prefs.setHostUser(itIsHost) }
                AppSarau.prefs.setUser(itUser.email)
                if (AppSarau.prefs.getUser().isNotEmpty() && AppSarau.prefs.getHostUser()) {
                    viewModel.saveHostToDB(itUser.asHost())
                } else if (AppSarau.prefs.getUser().isNotEmpty() && !AppSarau.prefs.getHostUser()) {
                    viewModel.saveUserToDB(itUser)
                }
            }
        }

        viewModel.savedHostIsSuccessFul.observe(this){
            if (it) navigateToMainHostActivity().also {
                binding.inclProgressBar.cvProgressBar.visibility = View.GONE
            }
        }

        viewModel.savedUserIsSuccessFul.observe(this){
            if (it) navigateToMainUserActivity().also {
                binding.inclProgressBar.cvProgressBar.visibility = View.GONE
            }
        }

    }

    private fun startSignIn() {
        val googleConfig =
            GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN).requestIdToken(
                getString(
                    com.firebase.ui.auth.R.string.default_web_client_id
                )
            )
                .requestEmail()
                .build()

        val googleClient = GoogleSignIn.getClient(this, googleConfig)

        val signInIntent = googleClient.signInIntent
        launcher.launch(signInIntent)

    }

    private fun navigateToMainHostActivity() {
        val intent = Intent(baseContext, MainHostActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun navigateToMainUserActivity() {
        val intent = Intent(baseContext, MainUserActivity::class.java)
        startActivity(intent)
        finish()
    }

/*    private fun onSignInResult(result: ActivityResult) {
        val response = result.data
        if (result.resultCode == RESULT_OK) {
            // Successfully signed in
            val task = GoogleSignIn.getSignedInAccountFromIntent(response)
            try {
                val account = task.getResult(ApiException::class.java)
                val credential = GoogleAuthProvider.getCredential(account.idToken, null)

                FirebaseAuth.getInstance().signInWithCredential(credential).addOnCompleteListener {
                    if (it.isSuccessful) {
                        Toast.makeText(
                            this,
                            "Log In ${account.email} successful",
                            Toast.LENGTH_SHORT
                        ).show()

                    } else {
                        // Sign in failed
                        Toast.makeText(this, "Sign In Failed", Toast.LENGTH_LONG).show()
                    }

                }
            } catch (e: ApiException) {
                Toast.makeText(this, "Sign In Failed", Toast.LENGTH_LONG).show()
            }

        } else {
            // Sign in failed
            Toast.makeText(this, "Sign In Failed", Toast.LENGTH_LONG).show()
        }
    }*/
}