package edu.uoc.bsolee.sarau.ui.login

import androidx.activity.result.ActivityResult
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.common.api.ApiException
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.model.Host
import edu.uoc.bsolee.sarau.model.User
import edu.uoc.bsolee.sarau.repository.AuthRepository
import edu.uoc.bsolee.sarau.repository.HostRepository
import edu.uoc.bsolee.sarau.repository.UserRepository

class LoginViewModel(
    private val authRepository: AuthRepository,
    private val hostRepository: HostRepository,
    private val userRepository: UserRepository
) : ViewModel() {

    val userLiveData = authRepository.getUserLiveData()

    private val _fragment = MutableLiveData(LoginActivity.USER_LOGIN_FRAGMENT)
    val fragment: LiveData<Int>
        get() = _fragment

    private val _savedHostIsSuccessFul = hostRepository.getSaveHostIsSuccessful()
    val savedHostIsSuccessFul: LiveData<Boolean>
        get() = _savedHostIsSuccessFul

    private val _savedUserIsSuccessFul = userRepository.getSaveUserIsSuccessful()
    val savedUserIsSuccessFul: LiveData<Boolean>
        get() = _savedUserIsSuccessFul


    private val _errorMessage = MutableLiveData(-1)
    val errorMessage: LiveData<Int>
        get() = _errorMessage

    private val _signIn = MutableLiveData(false)
    val signIn: LiveData<Boolean>
        get() = _signIn

    fun onClickSignIn() {
        _signIn.value = true
    }

    fun onClickSignInDone() {
        _signIn.value = false
    }

    fun onSignInResult(result: ActivityResult) {
        val response = result.data
        if (result.resultCode == AppCompatActivity.RESULT_OK) {
            // Successfully signed in
            val task = GoogleSignIn.getSignedInAccountFromIntent(response)
            try {
                val account = task.getResult(ApiException::class.java)
                authRepository.login(account)
            } catch (e: ApiException) {
                _errorMessage.value = R.string.sign_in_failed
            }
        } else {
            // Sign in failed
            _errorMessage.value = R.string.sign_in_failed
        }
    }

    private val _isHostUser = MutableLiveData(false)
    val isHostUser: LiveData<Boolean>
        get() = _isHostUser

    fun onClickLogInAsHost() {
        _isHostUser.value = true
        _fragment.value = LoginActivity.HOST_LOGIN_FRAGMENT
    }

    fun onClickLogInAsUser() {
        _isHostUser.value = false
        _fragment.value = LoginActivity.USER_LOGIN_FRAGMENT
    }

    fun saveHostToDB(host: Host) {
        hostRepository.saveHost(host)
    }

    fun saveUserToDB(user: User) {
        userRepository.saveUser(user)
    }


}