package edu.uoc.bsolee.sarau.ui.splash

import android.annotation.SuppressLint
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.view.animation.Animation
import android.view.animation.AnimationUtils
import androidx.appcompat.app.AppCompatActivity
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.databinding.ActivitySplashScreenBinding
import edu.uoc.bsolee.sarau.ui.host.main_host.MainHostActivity
import edu.uoc.bsolee.sarau.ui.login.LoginActivity
import edu.uoc.bsolee.sarau.ui.user.main_user.MainUserActivity

@Suppress("DEPRECATION")
@SuppressLint("CustomSplashScreen")
class SplashScreenActivity : AppCompatActivity() {

    // View Binding
    private lateinit var binding: ActivitySplashScreenBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivitySplashScreenBinding.inflate(layoutInflater)
        val view = binding.root
        setContentView(view)

        // Hide the status bar.
        window.decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_FULLSCREEN

        // Logo Zoom in
        val zoomInAnim = AnimationUtils.loadAnimation(this, R.anim.zoom_in)
        binding.ivLogo.startAnimation(zoomInAnim)

        zoomInAnim.setAnimationListener(object : Animation.AnimationListener {
            override fun onAnimationStart(p0: Animation?) {}
            override fun onAnimationEnd(p0: Animation?) {
                if (AppSarau.prefs.getUser().isNotEmpty() && AppSarau.prefs.getHostUser()) {
                    navigateToMainHostActivity()
                } else if (AppSarau.prefs.getUser().isNotEmpty() && !AppSarau.prefs.getHostUser()) {
                    navigateToMainUserActivity()
                } else {
                    navigateToLoginActivity()
                }
            }

            override fun onAnimationRepeat(p0: Animation?) {}

        })
    }

    private fun navigateToMainHostActivity() {
        val intent = Intent(baseContext, MainHostActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun navigateToMainUserActivity() {
        val intent = Intent(baseContext, MainUserActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun navigateToLoginActivity() {
        val intent = Intent(baseContext, LoginActivity::class.java)
        startActivity(intent)
        finish()
    }


}