package edu.uoc.bsolee.sarau.ui.user.booking_detail

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.model.SelectedTicket
import edu.uoc.bsolee.sarau.repository.EventRepository
import edu.uoc.bsolee.sarau.repository.UserRepository

class BookingDetailViewModel(
    private val eventRepository: EventRepository,
    private val userRepository: UserRepository
) : ViewModel() {

    private val _booking = userRepository.getBookingFromIdLiveData()
    val booking: LiveData<Booking> get() = _booking

    private val _event = eventRepository.getEventLiveData()
    val event: LiveData<Event> get() = _event

    private var _bookingQR = ""
    val bookingQR: String get() = _bookingQR

    private val _bookedTickets = MutableLiveData<List<SelectedTicket>>()
    val bookedTickets: LiveData<List<SelectedTicket>> get() = _bookedTickets


    fun setBookingQR(qr: String) {
        _bookingQR = qr
        userRepository.getBookingFromId(qr)
    }

    fun getEventInfo(eventId: String) {
        eventRepository.getEventFromId(eventId)
    }

    fun getBookedTickets(booking: Booking) {
        _bookedTickets.value = booking.selectedTickets
    }

    private val _accountCopied = MutableLiveData(false)
    val accountCopied: LiveData<Boolean> get() = _accountCopied

    fun onAccountCopied() {
        _accountCopied.value = true
    }

    fun onAccountCopiedDone() {
        _accountCopied.value = false
    }


}