package edu.uoc.bsolee.sarau.ui.user.main_user

import android.content.Intent
import android.os.Bundle
import android.transition.TransitionInflater
import android.view.*
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.recyclerview.widget.LinearLayoutManager
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.adapters.EventUserAdapter
import edu.uoc.bsolee.sarau.databinding.FragmentEventsUserBinding
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.ui.user.my_account.UserAccountActivity


class EventsUserFragment : Fragment() {
    companion object {
        @JvmStatic
        fun newInstance() = EventsUserFragment()
    }

    // ViewModel
    private val viewModel: MainUserViewModel by activityViewModels { MainUserViewModelFactory() }

    // View Binding
    private var _binding: FragmentEventsUserBinding? = null
    private val binding: FragmentEventsUserBinding
        get() = _binding!!

    // adapter
    private val eventAdapter = EventUserAdapter(this, EventUserAdapter.EventUserClickListener {
        openDetail(it)
    })

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setHasOptionsMenu(true)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
        exitTransition = inflater.inflateTransition(R.transition.fade)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentEventsUserBinding.inflate(inflater, container, false)
        val view = binding.root

        initUi()
        initObservers()
        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        (activity as AppCompatActivity).supportActionBar?.setDisplayHomeAsUpEnabled(false)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        inflater.inflate(R.menu.user_main_menu, menu)
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem) = when (item.itemId) {
        R.id.my_account -> {
            navigateToMyAccount()
            true
        }
        else -> {
            super.onOptionsItemSelected(item)
        }
    }

    private fun initUi() {
        activity?.title = getString(R.string.events_list)
        binding.inclProgressBar.cvProgressBar.visibility = View.VISIBLE
        // recyclerview
        binding.rvActiveEvents.layoutManager = LinearLayoutManager(requireContext())
        binding.rvActiveEvents.adapter = eventAdapter
    }

    private fun initObservers() {
        viewModel.nextEvents.observe(viewLifecycleOwner) {
            eventAdapter.submitList(it.filter { event -> event.published })
            binding.inclProgressBar.cvProgressBar.visibility = View.GONE
        }

    }

    private fun openDetail(event: Event) {
        viewModel.setEventId(event.id)
        viewModel.setFragment(MainUserActivity.EVENT_DETAIL_FRAGMENT)
    }

    private fun navigateToMyAccount() {
        val intent = Intent(requireContext(), UserAccountActivity::class.java)
        startActivity(intent)
    }


}