package edu.uoc.bsolee.sarau.ui.user.my_account

import android.graphics.Bitmap
import android.net.Uri
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.User
import edu.uoc.bsolee.sarau.repository.AuthRepository
import edu.uoc.bsolee.sarau.repository.UserRepository

class UserAccountViewModel(
    private val authRepository: AuthRepository,
    private val userRepository: UserRepository
) : ViewModel() {

    private val _user = userRepository.getUserLiveData()
    val user: LiveData<User> get() = _user

    private val _bookings = userRepository.getBookingsFromUserLiveData()
    val bookings: LiveData<List<Booking>> get() = _bookings

    private val _userSavedSuccessful = userRepository.getSaveUserIsSuccessful()
    val userSavedSuccessful: LiveData<Boolean> get() = _userSavedSuccessful

    // Local Uri Image
    private val _uriImage = MutableLiveData<Uri>()
    val uriImage: LiveData<Uri>
        get() = _uriImage

    private val _isLoggedOutDone = authRepository.getLoggedOutLiveData()
    val isLoggedOutDone: LiveData<Boolean>
        get() = _isLoggedOutDone

    init {
        getCurrentUser()
        getBookings()
    }

    fun updateUserToDB(userEdited: User) {
        userRepository.saveOrUpdateUser(userEdited)
    }

    fun updateImageAndSaveUser(userEdited: User, bitmap: Bitmap){
        userRepository.updateImageAndSave(userEdited, bitmap)
    }

    fun getCurrentUser() {
        userRepository.getUser(AppSarau.prefs.getUser())
    }

    private fun getBookings() {
        userRepository.getBookingsFromUser()
    }

    fun selectImage(uri: Uri?) {
        uri?.let { _uriImage.value = it }
    }

    fun logout() {
        authRepository.logOut()
    }

    fun clearData() {
        AppSarau.prefs.clear()
    }


}